@@ -16,8 +16,8 @@ proc Mh_Init {} {
     global exmh nmh
     MhParseProfile
 
-    set nmh 0
-    catch {string match *group* [MhExec repl -help] } nmh
+    catch {MhExec repl -help} output
+    set nmh [string match *group* $output]
 
     # set $exmh(mh_vers) to a pretty-printable string...
     set exmh(mh_vers) "unknown"
@@ -440,6 +440,7 @@ proc MhReadSeqs {folder seqsvar} {
 		Exmh_Debug MhReadSeqs Reading $filename
 		set old [read $in]
 		close $in
+		set mhPriv(otherpriv) {}
 		foreach line [split $old \n] {
 		    if {$line != {}} {
 			if {[regexp {^([^:]*):\s*(.*)$} $line foo tag msgids]} {
@@ -517,7 +518,6 @@ proc MhReadSeqs {folder seqsvar} {
 
 proc MhGetSeqCache {folder seq } {
     global mhPriv
-    Exmh_Debug "MhGetSeqCache $folder $seq"
     set seqlist ""
     if {[info exists mhPriv(pubseq,$folder,$seq)]} {
         set seqlist $mhPriv(pubseq,$folder,$seq)
@@ -547,6 +547,10 @@ proc MhSeqExpand { folder sequence } {
     set seq {}
     set rseq {}
     foreach range [split [string trim $sequence]] {
+	if ![regexp {^[0-9]+(-[0-9]+)?$} $range] {
+	    # just ignore anything bogus
+	    continue;
+	}
 	set parts [split [string trim $range] -]
 	if {[llength $parts] == 1} {
 	    lappend seq $parts
@@ -621,11 +625,11 @@ proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
 		if {$mhPriv(mode,$thisseq) == "public"} {
 		    if {![regexp {^ *$} $seqs($thisseq)]} {
 			if [regexp -- {-} $seqs($thisseq)] {
-			    set seq $seqs($thisseq)
+			    set realseq $seqs($thisseq)
 			} else {
-			    set seq [MhSeqMake $seqs($thisseq)]
+			    set realseq [MhSeqMake $seqs($thisseq)]
 			}
-			puts $out "$thisseq: $seq"
+			puts $out "$thisseq: $realseq"
 		    }
 		}
 	    }
@@ -650,11 +654,11 @@ proc Mh_SequenceUpdate { folder how seq {msgids {}} {which public}} {
 		if {[string compare $mhPriv(mode,$thisseq) "private"] == 0} {
 		    if {![regexp {^ *$} $seqs($thisseq)]} {
 			if [regexp -- {-} $seqs($thisseq)] {
-			    set seq $seqs($thisseq)
+			    set realseq $seqs($thisseq)
 			} else {
-			    set seq [MhSeqMake $seqs($thisseq)]
+			    set realseq [MhSeqMake $seqs($thisseq)]
 			}
-			puts $out "atr-$thisseq-$mhProfile(path)/$folder: $seq"
+			puts $out "atr-$thisseq-$mhProfile(path)/$folder: $realseq"
 		    }
 		}
 	    }
@@ -1214,6 +1218,12 @@ proc Mh_FindFile { filename } {
 }
 # exmh-2.5 APIs
 # Mh_ClearCur 
-# Mh_MarkSeen
-# Mh_MarkUnseen
 # Mh_Unseen 
+
+proc Mh_MarkSeen {folder ids} {
+    global mhProfile
+    Seq_Del $folder $mhProfile(unseen-sequence) $ids
+}
+proc Mh_MarkUnseen {folder ids} {
+    Seq_Add $folder $mhProfile(unseen-sequence) $ids
+}
