@@ -202,12 +202,15 @@ with the machine-readable part (part 2) of multipart/report messages."}
 {mime(mime_alternative_prefs) mime_alternative_prefs {} 
 {multipart/alternative precedence} 
 "If a message contains a MIME part of type multipart/alternative, exmh 
-will select one of the alternatives for display. You can control the 
-order of preference of the various alternatives. The most common use 
-for this is to establish the precedence of text/plain over text/html 
-for exmh users who do not use the in-line html viewer for showing 
-text/html content. If no value of mime_alternative_prefs is provided, 
-exmh will display the last alternative that it is capable of displaying."}
+will select the last supported type for display as indicated by 
+RFC2046. You can override this behaviour and control the order of 
+preference of the various alternatives. The most common use for this 
+is to establish a precedence of text/plain over text/html.
+
+The following line makes exmh display the text/plain part in favour 
+of the text/html part if both are present in a multipart/alternative:
+
+text/plain text/html"}
     }
     set i 0
     foreach char {A B C D E F G H I J K L M N O P Q R S T U V W X Y Z \
@@ -637,7 +640,7 @@ proc MimeSetDisplayFlag {part} {
 	}
 	if {[info exists mimeHdr($part,hdr,content-disposition)]} {
 	    set disp [split $mimeHdr($part,hdr,content-disposition) \;]
-	    if [regexp {ancillary} [lindex $disp 0]] {
+	    if [regexp {ancillary|attachment} [lindex $disp 0]] {
 		set mimeHdr($part,display) 0
 		return
 	    }
@@ -1309,14 +1312,25 @@ proc MimeSortHeaders { a b } {
 
   # Modification from Tom Lane so that headers that appear in the
   # Header-Display .mh_profile setting appear first, before
-  # stray headers that are not surpressed with Header-Supress
-
-  set apos [lsearch -regexp $mhProfile(header-display) $a]
-  if {$apos < 0} { set apos 100000 }
-  set bpos [lsearch -regexp $mhProfile(header-display) $b]
-  if {$bpos < 0} { set bpos 100000 }
-
+  # stray headers that are not suppressed with Header-Supress
+  # Further change - the old lsearch -regexp was completely
+  # wrong.  The header-display is a list of regular expressions
+  # to test against the header value.  You can't do that with lsearch.
 
+  set apos 0
+  foreach item $mhProfile(header-display) {
+      if {[regexp -nocase ^${item}\$ $a]} {
+          break
+      }
+      incr apos
+  }
+  set bpos 0
+  foreach item $mhProfile(header-display) {
+      if {[regexp -nocase ^${item}\$ $b]} {
+          break
+      }
+      incr bpos
+  }
    return [expr {$apos - $bpos}]
 }
 proc MimeShowMinHeaders {tkw part inlin} {
@@ -2134,7 +2148,20 @@ proc MimeParseSingle {tkw part fileIO } {
 		[expr {$mime(enabled) && 
 		       [info exists mimeHdr(0=1,hdr,content-type)]}]
 	}
-
+        #
+        # Skip through blank lines looking for the start of a PGP message
+        # Old code used to just consume a single line
+        # (Based on a patch from Joel Hatton)
+        #
+        set firstLine ""
+        while {! [eof $fileIO]} {
+            set firstLinePosition [tell $fileIO]
+            gets $fileIO firstLine
+            if { ! [regexp {(?n)^\s*$} $firstLine ] } {
+              break
+            }
+        }
+        if [regexp $miscRE(beginpgp) $firstLine] { set mimeHdr($part,decode) 1 }
     } else {
 	Exmh_Status "Warning - no headers" warn
 	set firstLine $line
@@ -2142,11 +2169,6 @@ proc MimeParseSingle {tkw part fileIO } {
 	set mimeHdr($part,encoding) [set encoding 8bit]
 	set mimeHdr($part,params) {}
     }
-    if {![info exists firstLine]} {
-	set firstLinePosition [tell $fileIO]
-	gets $fileIO firstLine
-	if [regexp $miscRE(beginpgp) $firstLine] { set mimeHdr($part,decode) 1 }
-    }
     if {$numBytes >= 0} {
 	if {$fast && [string compare $type text/plain] == 0 &&
 	    [regexp {[78]bit} $encoding] &&
