@@ -18,246 +18,6 @@
 # Pgp_Init is in extrasInit.tcl
 # to avoid auto-loading this whole file.
 
-# $Log: pgpMain.tcl,v $
-# Revision 1.23  2001/07/11 18:13:59  welch
-# ftp.expect.MASTER:
-# inc.expect.MASTER: Changed to a #!/bin/sh header with exec hack
-# to be able to insert the -- flag to expect
-# install.tcl: Changed the expect patching to match the above change
-# lib/pgpMain.tcl:
-# lib/mime.tcl: Changed MsgTextHighlight to Msg_TextHighlight
-# lib/msgShow.tcl: Added Msg_HighlightInit
-# lib/sedit.tcl: Added SeditBeautify from John Beck
-# lib/seditBind.tcl: Added "highlight" virtual function to sedit bindings
-# lib/app-defaults: Added <Control-l> binding to sedit to beautify it.
-#
-# Revision 1.22  2001/07/09 16:45:33  welch
-# Jumbo commit - I think I forgot to commit a few things from the
-# aborted 2.4 release as well.  The main thing is the addition of
-# message highlighting.
-#
-# Makefile:
-# exmh.README:
-# exmh.CHANGES:
-# lib/html/exmh.README.html:
-# lib/html/index.html:
-# lib/html/software.html:
-# version.sed:
-# exmh.install: Changing version number to 2.5
-# exmh-bg.MASTER:
-# exmh.MASTER: Fixed initialization of exmh(userLibrary)
-# inc.expect.MASTER:  Fixed !# line so it gets installed right
-# lib/app-defaults-color: Added resources for message highlighting
-# based on the jcl-beautify code.
-# lib/autorefile.tcl: Contributed code by John Carroll
-# lib/fdispColor.tcl:
-# lib/fdisp.tcl: Added use of c_unseenBg and c_movedFg color resources
-# for further refinement of unseen and moved messages.
-# lib/inc.tcl: Cleanup of Inc_Expect
-# lib/mailcap.tcl: Fixed bug in mailcap parsing code that didn't quote
-# & in rules, leading to message corruption in SaveAttachments code.
-# lib/mh.tcl: minor tweak to variable unset
-# lib/mime.tcl: Added highlightText option and calls to Msg_TextHighlight.
-# *  lib/msgShow.tcl: Added Msg_TextHighlight and the jcl-beautify code,
-# with minor changes to separate the bug reporting header highlighting
-# into a different hook.  There is now a "message show" hook that
-# operates on the whole message, a "message highlight" hook that
-# only operates on the text regions of a message.
-# lib/pgpMain.tcl: Added call to Msg_TextHighlight.
-# lib/pop.tcl: Fixed Pop_Dialog so it works right with multiple hosts.
-#
-# Revision 1.21  2000/06/16 18:16:26  valdis
-# Various PGP fixes...
-#
-# Revision 1.20  2000/04/22 10:34:49  gruber
-# Handle PGP messages without Version: xxx
-#
-# Revision 1.19  1999/10/27 14:59:27  kchrist
-# Quick application/pgp bug fix.
-#
-# Revision 1.18  1999/10/25 15:38:39  kchrist
-# Added a dropKeys pattern to pgpGPG.tcl.
-#
-# Simplified PGP GUI by removing "detached" signature option. Problem
-# was that MIME+standard includes a copy of the message being signed
-# in the signature attachement. What really should be used is
-# MIME+detached. Decided to overload the meaning of "standard". If
-# the format is plain, standard means "binary". If the format is
-# anything else, standard means "detached". Less flexibility but
-# better chances of "doing the right thing".
-#
-# Revision 1.17  1999/10/07 14:29:28  kchrist
-# Changed quote file name documentation string to remove ambiguity regarding
-# the location of this file.
-#
-# Fixed a PGP bug when sign-encrypting a MIME message.
-#
-# Revision 1.16  1999/09/30 14:55:20  kchrist
-# One more fix to the cmd_User issue.
-#
-# Revision 1.15  1999/09/30 03:51:07  kchrist
-# pgp($v,cmd_Beauty) was getting in the way of pgp($v,cmd_User) for
-# v=gpg so I had to rearrange things a bit.
-#
-# Revision 1.14  1999/09/27 23:18:45  kchrist
-# More PGP changes. Consolidated passphrase entry to sedit field or
-# pgpExec routine. Made the pgp-sedit field aware of pgp(keeppass)
-# and pgp(echopass). Moved pgp(keeppass), pgp(echopass) and
-# pgp(grabfocus) to PGP General Interface. Fixed a minor bug left
-# over from my previous GUI changes. Made pgp-sedit field appear and
-# disappear based on its enable preference setting.
-#
-# Revision 1.13  1999/09/22 16:36:44  kchrist
-# Changes made to support a different structure under the PGP Crypt... button.
-# Instead of an ON/OFF pgp($v,sign) variable now we use it to specify
-# the form of the signature (none, standard, detached, clear, or w/encrypt).
-# Code changed in several places to support this new variable definition.
-#
-# Updated Sedit.html to include a description of the new interface.
-#
-# Revision 1.12  1999/08/24 15:51:07  bmah
-# Patch from Kevin Christian to make email PGP key queries work, and
-# to make key attachment RFC 2015 compliant.
-#
-# Revision 1.11  1999/08/22 19:21:33  bmah
-# Remove More...->old PGP->Encrypt menu.  Getting this to work will be
-# really hard and there doesn't seem to be a lot of demand for it, given
-# the other PGP functionality.
-#
-# Revision 1.10  1999/08/18 15:52:37  bmah
-# Commit modification to seditpgp user interface, and fix a couple of
-# bugs.
-#
-# Revision 1.9  1999/08/13 00:39:05  bmah
-# Fix a number of key/passphrase management problems:  pgpsedit now
-# manages PGP versions, keys, and passphrases on a per-window
-# basis.  Decryption now works when no passphrases are cached.
-# One timeout parameter controls passphrases for all PGP
-# versions.  seditpgp UI slightly modified.
-#
-# Revision 1.8  1999/08/11 06:16:39  bmah
-# Properly decode multipart/signed PGP messages whose boundary strings
-# contain characters that are interpreted as special regexp characters.
-#
-# Revision 1.7  1999/08/05 15:46:29  bmah
-# Fix seditpgp key-changing button label code (update properly when
-# user changes her key or changes the PGP version).
-#
-# Revision 1.6  1999/08/04 22:43:39  cwg
-# Got passphrase timeout to work yet again
-#
-# Revision 1.5  1999/08/04 00:21:57  iko
-# Fix multiple Mime-Version: with pgp signed messages
-#
-# Revision 1.4  1999/08/03 18:06:43  bmah
-# Permit user to cancel selection of a private key for signing (affects
-# the "Choose Key..." dialog in sedit and the button at the bottom of the
-# pgpsedit window).
-#
-# Revision 1.3  1999/08/03 16:31:43  cwg
-# Display the body of a message which fails to be decoded by PGP.
-#
-# Revision 1.2  1999/08/03 04:05:55  bmah
-# Merge support for PGP2/PGP5/GPG from multipgp branch.
-#
-# Revision 1.8  1999/05/06 15:36:41  cwg
-# If there's a PGP error while processing the message, show the message raw.
-#
-# Revision 1.7  1999/05/05 14:58:10  cwg
-# Modifed Jan Peterson's code to make better use of screen real estate.
-#
-# Revision 1.6  1999/04/30 19:09:00  cwg
-# Jan Peterson's multiple PGP key patch
-#
-# Revision 1.5  1999/04/20 21:46:17  cwg
-# Is CVS working now?
-#
-# Revision 1.4  1999/04/15 23:41:59  cwg
-# Make the crypt menu values be per-window instead of global.
-#
-# Revision 1.3  1999/03/29 20:49:20  cwg
-# If doing a plain signature, disable usage of the pgp(mime) flag.
-#
-# Revision 1.2  1999/03/26 08:41:55  cwg
-# Changes to PGP interface to use preferences variables instead of
-# message headers.  Also, reorganize the "PGP..." menu and rename it
-# "Crypt..."
-#
-# See the "PGP interface" preferences page for more info.
-#
-# Revision 1.1  1998/05/05 17:55:37  welch
-# Initial revision
-#
-# Revision 1.1  1998/05/05 17:42:59  welch
-# Initial revision
-#
-# Revision 1.16  1998/01/22  00:47:01  bwelch
-#     Fixed Pgp_Setup to use /dev/tty instead of /dev/console
-#
-# Revision 1.15  1997/12/22  20:53:46  bwelch
-# File_Delete
-#
-# Revision 1.14  1997/07/25  17:14:09  bwelch
-# Trapped error messages from PGP xterms.
-#
-# Revision 1.13  1997/07/12  23:06:28  bwelch
-#     Added expecttk support for PGP handling.
-#     Fixed recursive failures with some PGP messages.
-#
-# Revision 1.12  1997/06/03  18:37:00  bwelch
-# Added doinc flag to Inc_Presort
-# Major cleanup
-#
-# Revision 1.11  1997/01/25  06:21:16  bwelch
-# Added Quote module support
-#
-# Revision 1.10  1997/01/25  05:34:08  bwelch
-#     Added Pgp_GetTextAttributes
-# Changed display of PGP messages
-#
-# Revision 1.9  1996/12/21  00:58:08  bwelch
-# Fixed PGP menu to support both email and www key fetching.
-# Guard against missing signature parts in multipart/signed
-#
-# Revision 1.8  1996/12/02  21:11:11  bwelch
-# Moved New Key button to be inside the help documents.
-#
-# Revision 1.7  1996/12/01  20:16:47  bwelch
-# Added mutipart/security support.
-# (Ben Escoto and Chris Garrigues)
-#
-# Revision 1.6  1996/03/22  18:44:34  bwelch
-# Changed graphic part separator size in PGP from 5 to 6 to avoid
-#     downloading a new font for this case.
-#
-# Revision 1.5  1995/09/28  04:11:10  bwelch
-# Fixed "hasfcc" check in PGP.
-#
-# Revision 1.4  1995/06/30  18:32:40  bwelch
-# Upcase PGP mail headers
-#
-# Revision 1.3  1995/06/09  20:57:06  bwelch
-# Added ChoosePrivateKey
-#
-# Revision 1.2  1995/05/24  06:01:38  bwelch
-# Added Pgp_SetMyName to choose private key name
-#
-# Revision 1.1  1995/05/24  01:48:03  bwelch
-# Initial revision
-#
-# Revision 1.21  1995/04/15  18:17:01  welch
-# Introduced msg(path)
-#
-# Revision 1.20  1995/03/22  22:17:30  welch
-# Changed exmh.PGP.help to help.PGP
-#
-# Revision 1.19  1995/03/22  18:53:52  welch
-# More new code from Stefan
-#
-# Revision 1.1  1994/12/17  20:18:49  monnier
-# Initial revision
-#
-
 # Setup is invoked by the "Make Key" button on the PGP Setup Help page
 # It searches for versions on the system, which are supported by exmh.
 # Setup then configures the version found on your system.
@@ -369,7 +129,6 @@ proc Pgp_HelpOld {} {
 	Widget_AddBut .pgphelp.but setup "Make Key" [list Pgp_Setup]
     
 	set t [Widget_Text .pgphelp 30 -setgrid true]
-	Ftoc_ColorConfigure $t
 	$t insert insert "EXMH Version: $exmh(version)\n\n"
 	if [catch {open "$exmh(library)/help.PGP" r} in] {
 	    $t insert insert "Cannot find file exmh.PGP.help to display"
@@ -403,18 +162,25 @@ proc Pgp_ExmhEncrypt { v } {
 
     Exmh_Status "pgp -e $exmh(folder)/$msg(id)"
 
-#    set pgp(param,recipients) [lindex $pgp($v,myname) 0]
+    set pgp(param,recipients) [lindex $pgp($v,myname) 0]
     set id [SeditId $file]
     set pgp(encrypt,$id) 1;
     set pgp(sign,$id) "none";
     set pgp(format,$id) "pm";
+    set pgp(version,$id) $v
 
     Pgp_Process $v $file $tmpfile
 
+    unset pgp(param,recipients)
+    unset pgp(encrypt,$id)
+    unset pgp(sign,$id)
+    unset pgp(format,$id)
+    unset pgp(version,$id)
+
     Mh_Rename $tmpfile $file
 
     set msg(dpy) {}
-    MsgChange $msg(id)
+    Msg_Change $msg(id)
 
     return 1
 }
@@ -644,11 +410,12 @@ proc Pgp_Process { v srcfile dstfile } {
     
     set mimeheaders [lindex $allheaders 0]
     set mailheaders [lindex $allheaders 1]
-	
+
     if {[lsearch -glob $mimeheaders "content-type:*"] < 0} {
 	lappend mimeheaders "content-type: text/plain; charset=us-ascii"
+        lappend mimeheaders "content-disposition: inline"
     }
-    
+
     # if there is nothing to do, stop here
     if {!$pgp(encrypt,$id) && $pgp(sign,$id)=="none"} {
 	close $orig
@@ -678,10 +445,18 @@ proc Pgp_Process { v srcfile dstfile } {
     # setup the originator (if necessary)
     if {$pgp(sign,$id) != "none"} {
 	Exmh_Debug PGP signing
-	if [info exists pgp(param,originator)] {
-	    set originator [PgpMatch_Simple $pgp(param,originator) $pgp(secring)]
+	if { [set pgp($v,choosekey)] && [llength $pgp($v,privatekeys)]>1} {
+	    set originator [lindex [Pgp_ChoosePrivateKey $v \
+	        "Please select a $pgp($v,fullName) key to use for signing"] 0]
+	    if {[string length $originator] == 0} {
+		error "A valid key is required for signing."
+	    }
 	} else {
-	    set originator $pgp($v,myname,$id)
+	    if [info exists pgp(param,originator)] {
+		set originator [Pgp_Match_Simple $v $pgp(param,originator) $pgp(secring)]
+	    } else {
+		set originator $pgp($v,myname,$id)
+	    }
 	}
 	if {$pgp(format,$id) == "app"} {
 	    append typeparams "; x-originator=[string range [lindex $originator 0] 2 end]"
@@ -692,7 +467,7 @@ proc Pgp_Process { v srcfile dstfile } {
     if {$pgp(encrypt,$id) || $pgp(sign,$id) == "encryptsign"} {
 	Exmh_Debug PGP encrypting
 	if [info exists pgp(param,recipients)] {
-	    set ids [Pgp_Misc_Map id {PgpMatch_Simple $id $pgp($v,pubring)} \
+	    set ids [Pgp_Misc_Map key {Pgp_Match_Simple $v $key $pgp($v,pubring)} \
 		    [split $pgp(param,recipients) ","]]
 	} else {
 	    set hasfcc [expr {[lsearch -glob $mailheaders "fcc:*"] >= 0}]
@@ -717,14 +492,18 @@ proc Pgp_Process { v srcfile dstfile } {
 		[list "mime-version: 1.0"] \
 		$mimeheaders]
     } else {
-	set pgpheaders $mimeheaders
+	if {$pgp(format,$id) == "pm"} {
+	    set pgpheaders $mimeheaders
+	} else {
+	    set pgpheaders {}
+	}
     }
 
     # write the message to be encrypted
     set msgfile [Mime_TempFile "msg"]
     set msg [open $msgfile w 0600]
     foreach line $pgpheaders { puts $msg [Pgp_Misc_FixHeader $line] }
-    puts $msg ""
+    if {[llength $pgpheaders] > 0} { puts $msg "" }
     puts -nonewline $msg [read $orig]
     close $orig
     close $msg
@@ -740,17 +519,24 @@ proc Pgp_Process { v srcfile dstfile } {
 		    # Depending on format standard may mean different
 		    # things. It was decided to keep this ambiguity
 		    # internal instead of exporting it via the GUI.
-		    if {$pgp(format,$id) == "plain"} {
-			Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
-				$originator standard
-		    } else {
+		    if {$pgp(format,$id) == "pm"} {
 			Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
 				$originator detached
+		    } else {
+			Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
+				$originator standard
 		    }
 		}
 		clearsign {
-		    Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
+		    # There is only one correct way of signing 
+		    # multipart/signed messages and that is "detached".
+		    if {$pgp(format,$id) == "pm"} {
+			Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
+				$originator detached
+		    } else {
+			Pgp_Exec_Sign $pgp(version,$id) $msgfile $pgpfile \
 			    $originator clearsign
+		    }
 		}
 		encryptsign {
 		    Pgp_Exec_EncryptSign $pgp(version,$id) $msgfile $pgpfile \
@@ -1225,6 +1011,23 @@ proc Pgp_ShowMessage { tkw part } {
 	    }
 	} else {
 	    set action $mimeHdr($part,param,x-action)
+	    # For signed messages we expect 'x-action=signclear' or 
+	    # 'x-action=signbinary' but some mailers only use 
+	    # 'x-action=sign'. Since application/pgp never made it 
+	    # to a valid content-type is hard to tell who is right.
+	    if {"$action" == "sign"} {
+		if [regexp $miscRE(beginpgpclear) $firstLine] {
+		    set action signclear
+		    set mimeHdr($part,param,x-action) signclear
+		} else {
+		    set action signbinary
+		    set mimeHdr($part,param,x-action) signbinary
+		}		    
+	    } elseif {"$action" == "pgp-signed"} {
+		# Mutt propagates yet another x-action type
+		set action signclear
+		set mimeHdr($part,param,x-action) signclear
+	    }
 	}
     } else {
 	set action "keys-only"
